<?php
/**
 * Manage Clickio CDN cache
 */

namespace Clickio\CacheControl\Services;

use Clickio as org;
use Clickio\CacheControl as cc;
use Clickio\Utils\SafeAccess;

/**
 * Clickio cdn cache service
 *
 * @package CacheControl\Services
 */
class ClickIoCDN extends cc\ServiceBase
{

    /**
     * Label
     *
     * @var string
     */
    protected $label = "Clickio CDN";

    /**
     * Description
     *
     * @var string
     */
    protected $desc = "Purge Clickio CDN cache";

    /**
     * Cleaner url
     *
     * @var string
     */
    const CLEAN_URL = 'https://all.stage.clickio.com/clickioUtils/clearUrlSyncBatch/';

    /**
     * Interface method
     * For more information see method defenition
     *
     * @param array $urllist list of urls
     *
     * @return void
     */
    public function clear(array $urllist)
    {
        if (empty($urllist)) {
            return ;
        }

        $integration = org\Options::get('integration_scheme');

        $params = $this->getRequestParams($urllist);
        $data = wp_remote_post(static::CLEAN_URL, $params);

        $params['body'] = json_decode($params['body'], \JSON_PRETTY_PRINT, \JSON_UNESCAPED_SLASHES);
        $this->debug('Purge CDN cache', ["url" => static::CLEAN_URL, "params" => $params, "response" => $data]);
    }

    /**
     * Build request params
     *
     * @param array $urls list of urls to be purged
     *
     * @return array
     */
    protected function getRequestParams(array $urls): array
    {
        $headers = [
            'Content-Type' => 'application/json',
            'Authorization' => 'Bearer '.org\Options::getApplicationKey()
        ];

        $body = $this->buildBody($urls);

        $params = [
            'timeout' => 15,
            'user-agent' => 'WP',
            'followlocation' => true,
            'sslverify' => false,
            'headers' => $headers,
            'body' => wp_json_encode($body)
        ];
        return $params;
    }

    /**
     * Build http body
     *
     * @param array $urls list of urls to be purged
     *
     * @return array
     */
    protected function buildBody(array $urls): array
    {
        $body = [];

        foreach ($urls as $url) {
            if (empty($url)) {
                continue ;
            }
            $params = $this->getUrlRequestParams($url);
            $body = array_merge($body, $params);
        }
        return $body;
    }

    /**
     * Build url struct
     *
     * @param string $url single url
     *
     * @return array
     */
    protected function getUrlRequestParams(string $url): array
    {
        $urls = [];

        $parsed_url = wp_parse_url($url);
        $_server_host = SafeAccess::fromArray($_SERVER, 'HTTP_HOST', 'string', 'default');
        $host = SafeAccess::fromArray($parsed_url, 'host', 'string', $_server_host);
        $path = SafeAccess::fromArray($parsed_url, 'path', 'string', '/');
        $query = SafeAccess::fromArray($parsed_url, 'query', 'string', '');

        $purge_all = false;
        if ($query == 'purge_all') {
            $purge_all = true;
        }

        $urls[] = $this->_getUrlRequestParam($host, $path, $purge_all);

        if ($purge_all) {
            return $urls;
        }

        if (substr($path, -1)=== '/') {
            $path_without_slash = substr($path, 0, (strlen($path) - 1));
            if (!empty($path_without_slash)) {
                $urls[] = $this->_getUrlRequestParam($host, $path_without_slash);
            }
        }
        return $urls;
    }

    /**
     * Build url struct
     *
     * @param string $host FQDN
     * @param string $path url
     * @param bool $purge_all flag, purge all pages
     *
     * @return array
     */
    private function _getUrlRequestParam(string $host, string $path, bool $purge_all = false): array
    {
        $url_param = [
            'daemon' => org\Options::get("daemon_version", "Master"),
            'domain' =>  $host,
            'url' => $path,
            'desktop' => 1,
            'mobile' => 1
        ];

        if ($purge_all) {
            $url_param['url'] = '/';
            $url_param['canonical'] = org\Options::get("purge_canonical");
            $url_param['all'] = 1;
        }
        return $url_param;
    }
}