<?php
/**
 * Extra content wrapper
 */

namespace Clickio\ExtraContent;

use Clickio\ExtraContent\Interfaces\IExtraContentService;
use Clickio\PageInfo\Rules;
use Clickio\Utils\SafeAccess;

/**
 * Facade. Extra content wrapper
 *
 * @package ExtraContent
 */
final class ExtraContent
{

    /**
     * List of extra content services
     *
     * @var array
     */
    protected $services = [];

    /**
     * Extra content container
     *
     * @var array
     */
    protected $extra = [];

    /**
     * Extra content rules
     *
     * @var array
     */
    protected $rules = [];

    /**
     * Constructor
     *
     * @param array $services list of services FQCN
     */
    public function __construct(array $services)
    {
        $this->services = $services;
        $this->rules = (Rules::create())->getRule("extra");
    }

    /**
     * Get extra content from selected services
     *
     * @param bool $force ignore settings
     *
     * @return array
     */
    public function getExtraContent(bool $force = false): array
    {

        foreach ($this->services as $service) {
            if (!$service instanceof IExtraContentService) {
                continue ;
            }

            $key = $service->getName();
            $rules = SafeAccess::fromArray($this->rules, $key, 'array', []);
            $service->setRules($rules);
            $extra = $service->getExtraContent($force);

            if (!empty($extra)) {
                $this->extra[$key] = $extra;
            }
        }

        return $this->extra;
    }

    /**
     * Get service by name
     *
     * @param string $name service name
     *
     * @throws \Exception
     *
     * @return IExtraContentService
     */
    public function getService(string $name): IExtraContentService
    {
        if (array_key_exists($name, $this->services)) {
            return $this->services[$name];
        }
        throw new \Exception("Service $name doesn't exists");
    }

    /**
     * Factory method.
     * Creates self with all service
     *
     * @param array $services service list
     *
     * @return self
     */
    public static function create(array $services = []): self
    {
        if (empty($services)) {
            ExtraContentServiceFactory::discover();
            $services = ExtraContentServiceFactory::createAll();
        }
        return new static($services);
    }

    /**
     * Getter
     * Get all services
     *
     * @return array
     */
    public function getServices(): array
    {
        return $this->services;
    }
}