<?php

/**
 * Cache engine repository
 */

namespace Clickio\Prism\Cache;

use Clickio\Logger\Logger;
use Clickio\Options;
use Clickio\Prism\Cache\Engine\Files;
use Clickio\Prism\Cache\Interfaces\ICacheEngine;
use Clickio\Prism\Cache\Interfaces\ICacheRepo;
use Clickio\Utils\DeviceType;
use Clickio\Utils\SafeAccess;

/**
 * Cache engine repository
 *
 * @package Prism\Cache
 */
final class CacheRepo implements ICacheRepo
{
    /**
     * Cache engine instance
     *
     * @var ICacheEngine
     */
    protected $engine = null;

    /**
     * Single tone container
     *
     * @var self
     */
    protected static $inst = null;

    /**
     * Constructor
     *
     * @param ICacheEngine $cache cache engine
     */
    public function __construct(ICacheEngine $cache)
    {
        $this->engine = $cache;
    }

    /**
     * Singletone constructor
     *
     * @return self
     */
    public static function getInstance()
    {
        if (!static::$inst) {
            $lx_nocache = SafeAccess::fromArray($_GET, 'lx_nocache', 'string', '0');
            if (!empty(Options::get("cache")) && empty($lx_nocache)) {
                $lifetime = Options::get('cache_lifetime');
                $cache = CacheFactory::make(Files::class, [$lifetime]);
            } else {
                $cache = CacheFactory::make(Dummy::class);
            }
            static::$inst = new static($cache);
        }
        return static::$inst;
    }

    /**
     * Get value from cache
     *
     * @param string $url cache key
     *
     * @return string
     */
    public function get(string $url): string
    {
        $key = $this->getCacheKey($url);
        return $this->engine->get($key);
    }

    /**
     * Put data into storage
     *
     * @param string $url cache key
     * @param mixed $value data to be cached
     *
     * @return bool
     */
    public function set(string $url, $value): bool
    {
        $key = $this->getCacheKey($url);
        $log_name = SafeAccess::fromArray($_SERVER, 'HTTP_HOST', 'string', 'localhost');
        $logger = Logger::getLogger($log_name);
        $logger->debug("Cached", ["cache_key" => $key, 'url' => $url]);
        return $this->engine->add($key, $value);
    }

    /**
     * Purge cache for a key
     *
     * @param string $url cache key
     * @param bool $raw_url use $url or add key to $url
     *
     * @return bool
     */
    public function purge(string $url, bool $raw_url = false): bool
    {
        if (!$raw_url) {
            $key = $this->getCacheKey($url);
        } else {
            $key = $url;
        }
        return $this->engine->purge($key);
    }

    /**
     * Purge all
     *
     * @return bool
     */
    public function purgeAll(): bool
    {
        return $this->engine->purgeAll();
    }

    /**
     * Split prism, mobile, desktop caches
     *
     * @param string $url page url
     *
     * @return string
     */
    protected function getCacheKey(string $url): string
    {
        $isMobile = DeviceType::isMobile();
        $isPrism = false;

        $mobile_opt = Options::get('mobile');
        $cl_debug = SafeAccess::fromArray($_REQUEST, 'cl_debug', 'string', '');

        if ((!empty($mobile_opt) && $isMobile) || !empty($cl_debug)) {
            $isPrism = true;
        }
        $key = sprintf("%s:%d:%d", $url, $isPrism, $isMobile);
        return $key;
    }
}