<?php

/**
 * Http request wrapper
 */

namespace Clickio\Request;

use Clickio\Authorization\TokenAuthorizationManager;
use Clickio\Request\Interfaces\IHttpRequest;
use Clickio\Request\Interfaces\IHttpResponse;
use Clickio\Utils\Container;

/**
 * Http request wrapper
 *
 * @package Request
 */
class Request extends Container implements IHttpRequest
{
    /**
     * Timeout
     *
     * @var int
     */
    protected $timeout = 15;

    /**
     * User-Agent
     *
     * @var string
     */
    protected $ua= 'WP';

    /**
     * Async
     *
     * @var bool
     */
    protected $blocking = true;

    /**
     * Verify ssl
     *
     * @var bool
     */
    protected $sslverify = false;

    /**
     * Signed request
     *
     * @var bool
     */
    protected $signed = true;

    /**
     * Make GET request
     *
     * @param string $url request url
     * @param array $query url query string
     *
     * @return IHttpResponse
     */
    public function get(string $url, array $query = []): IHttpResponse
    {
        $params = $this->toArray();
        $query = http_build_query($query);
        if ($this->signed) {
            $auth = TokenAuthorizationManager::create();
            $token = $auth->generateToken($query);
            $url = sprintf("%s?%s&token=%s", $url, $query, $token);
        } else {
            $url = sprintf("%s?%s", $url, $query);
        }
        $resp = wp_remote_get($url, $params);

        if (is_wp_error($resp)) {
            throw new \Exception($resp->get_error_message());
        }

        return Response::create($resp);
    }

    /**
     * Make POST request
     *
     * @param string $url request url
     * @param array $body request body
     * @param array $extra extra body values
     *
     * @return IHttpResponse
     */
    public function post(string $url, array $body = [], array $extra = []): IHttpResponse
    {
        $params = $this->toArray();

        if ($this->signed) {
            $auth = TokenAuthorizationManager::create();
            $payload = json_encode($body);
            $token = $auth->generateToken($payload);
            $params['body'] = $extra;
            $params['body']['payload'] = $payload;
            $params['body']['token'] = $token;
        } else {
            $params['body'] = $body;
        }
        $resp = wp_remote_post($url, $params);

        if (is_wp_error($resp)) {
            throw new \Exception($resp->get_error_message());
        }

        return Response::create($resp);
    }

    /**
     * Copy self value into array
     *
     * @return array
     */
    public function toArray()
    {
        $arr = parent::toArray();
        $ua = $arr['ua'];
        unset($arr['ua']);
        $arr['user-agent'] = $ua;
        return $arr;
    }
}