<?php

/**
 * Plugins utils
 */

namespace Clickio\Utils;

use Clickio\Integration\IntegrationManager;

/**
 * Plugins utils
 *
 * @package Utils
 */
class Plugins
{

    protected static $black_list = [
        'lazyload_videos_frontend',
        'redirect_canonical'
    ];
    /**
     * Get plugin info where callback is defined
     *
     * @param mixed $callback callback entity
     *
     * @return array
     */
    public static function getPluginByCallback($callback): array
    {
        if (!function_exists('get_plugins')) {
            include_once ABSPATH.'wp-admin/includes/plugin.php';
        }

        $method = static::_getMethodCallback($callback);

        if (!$method) {
            return [];
        }


        $file_path = $method->getFileName();

        if (strstr($file_path, WP_PLUGIN_DIR)) {
            foreach (get_plugins() as $plugin_file => $plugin_data) {
                if (!is_plugin_active($plugin_file)) {
                    continue ;
                }

                $plugin_dir = explode(DIRECTORY_SEPARATOR, plugin_basename($file_path))[0];
                if (strstr($plugin_file, $plugin_dir)) {
                    $plugin_data['PluginFile'] = $plugin_file;
                    return $plugin_data;
                }
            }
        }
        return [];
    }

    /**
     * Get reflected callback
     *
     * @param mixed $callback callback entity
     *
     * @return mixed
     */
    private static function _getMethodCallback($callback)
    {
        $callable = $callback;
        if (is_string($callable)) {
            $callable = explode("::", $callback);
        }

        try{
            if (count($callable) > 1) {
                $cls = SafeAccess::fromArray($callable, 0, 'mixed', null);
                $meth = SafeAccess::fromArray($callable, 1, 'mixed', null);
                if (!empty($cls) && !empty($meth)) {
                    $ref_cls = new \ReflectionClass($cls);
                    $method = $ref_cls->getMethod($meth);
                } else {
                    $method = null;
                }
            } else {
                $func = SafeAccess::fromArray($callable, 0, 'mixed', null);
                if (!empty($func)) {
                    $method = new \ReflectionFunction($func);
                } else {
                    $method = null;
                }
            }
        } catch (\Exception $e) {
            $method = null;
        }
        return $method;
    }

    /**
     * Check if callback is safe
     * This not gurantied that callback will not trigger some error
     *
     * @param mixed $callback callback entity
     *
     * @return bool
     */
    public static function pluginCallbackIsSafe($callback): bool
    {
        if ($callback instanceof \Closure) {
            return false;
        }

        $method = static::_getMethodCallback($callback);
        if (!$method || in_array($method->getName(), static::$black_list)) {
            return false;
        }

        $props = $method->getParameters();
        foreach ($props as $param) {
            if ($param->hasType()) {
                return false;
            }

            if (!$param->isOptional()) {
                return false;
            }
        }
        return true;
    }

    /**
     * Safe check is plugin  active
     *
     * @param string $id plugin id
     *
     * @return bool
     */
    public static function pluginIsActive(string $id): bool
    {
        $plugins = static::getPlugins();
        return array_key_exists($id, $plugins) && \is_plugin_active($id);
    }

    /**
     * Get available plugins
     *
     * @return array
     */
    public static function getPlugins(): array
    {
        if (!function_exists('get_plugins') || !function_exists('is_plugin_active')) {
            include_once ABSPATH.'wp-admin/includes/plugin.php';
        }
        $plugins = \get_plugins();
        return $plugins;
    }
}